<?php
namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use SM\Traits\TQuery;

class DashboardFactory {

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
      $this->connection = $entityManager->getConnection();
      $this->entityManager = $entityManager;
    }

    public function getApprovedContracts($parametros) {

        $sqlAlter = "ALTER SESSION SET NLS_DATE_FORMAT = 'DD/MM/YYYY HH24:MI:SS'";

        $std = $this->connection->prepare($sqlAlter);
        $std->execute();

        $sql = "SELECT GRAF.*
                  FROM (SELECT FCT_CONTRATO.NRTIPOORCAMENTO, FCT_TIPOORCAMENTO.NMTIPOORCAMENTO, COUNT(FCT_APROVACONTRATO.NRSEQAPROVCONT) AS QUANTIDADE
                          FROM FCT_APROVACONTRATO
                          JOIN FCT_CONTRATO
                           ON FCT_CONTRATO.NRSEQCONTRATO = FCT_APROVACONTRATO.NRSEQCONTRATO
                          AND FCT_CONTRATO.NRORG = FCT_APROVACONTRATO.NRORG
                          JOIN FCT_TIPOORCAMENTO
                            ON FCT_TIPOORCAMENTO.NRTIPOORCAMENTO = FCT_CONTRATO.NRTIPOORCAMENTO
                           AND FCT_TIPOORCAMENTO.NRORG = FCT_CONTRATO.NRORG
                         WHERE FCT_APROVACONTRATO.DTAPROVCONTR BETWEEN NVL(:DTINICIO, SYSDATE-30) AND NVL(:DTFINAL, SYSDATE)
                         AND FCT_APROVACONTRATO.IDAPROCCONTR = 'A'
                         AND FCT_CONTRATO.IDATIVO = 'S'
                         GROUP BY FCT_CONTRATO.NRTIPOORCAMENTO, FCT_TIPOORCAMENTO.NMTIPOORCAMENTO
                         ORDER BY QUANTIDADE DESC) GRAF
                         WHERE ROWNUM <= 7";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
        return $std->fetchAll();
    }

    public function getTerminatedContracts($parametros) {

        $sqlAlter = "ALTER SESSION SET NLS_DATE_FORMAT = 'DD/MM/YYYY HH24:MI:SS'";

        $std = $this->connection->prepare($sqlAlter);
        $std->execute();

        $sql = "SELECT GRAF.*
                  FROM (SELECT FCT_CONTRATO.NRTIPOORCAMENTO, FCT_TIPOORCAMENTO.NMTIPOORCAMENTO, COUNT(FCT_CONTRATO.NRSEQCONTRATO) AS QUANTIDADE
                          FROM FCT_CONTRATO
                          JOIN FCT_TIPOORCAMENTO
                            ON FCT_TIPOORCAMENTO.NRTIPOORCAMENTO = FCT_CONTRATO.NRTIPOORCAMENTO
                           AND FCT_TIPOORCAMENTO.NRORG = FCT_CONTRATO.NRORG
                         WHERE FCT_CONTRATO.DTRECISAOCONTR BETWEEN NVL(:DTINICIO, SYSDATE-30) AND NVL(:DTFINAL, SYSDATE)
                           AND FCT_CONTRATO.IDSITUACAOCONTRATO IN ('E', 'R')
                         GROUP BY FCT_CONTRATO.NRTIPOORCAMENTO, FCT_TIPOORCAMENTO.NMTIPOORCAMENTO
                         ORDER BY QUANTIDADE DESC) GRAF
                         WHERE ROWNUM <= 7";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
        return $std->fetchAll();
    }

    public function getRenegotiatedContracts($parametros) {

      $sqlAlter = "ALTER SESSION SET NLS_DATE_FORMAT = 'DD/MM/YYYY HH24:MI:SS'";

      $std = $this->connection->prepare($sqlAlter);
      $std->execute();

      $sql = "SELECT GRAF.*
                FROM (SELECT FCT_CONTRATO.NRTIPOORCAMENTO, FCT_TIPOORCAMENTO.NMTIPOORCAMENTO, COUNT(FCT_CONTRATO.NRSEQCONTRATO) AS QUANTIDADE
                        FROM FCT_CONTRATO
                        JOIN FCT_TIPOORCAMENTO
                          ON FCT_TIPOORCAMENTO.NRTIPOORCAMENTO = FCT_CONTRATO.NRTIPOORCAMENTO
                         AND FCT_TIPOORCAMENTO.NRORG = FCT_CONTRATO.NRORG
                       WHERE FCT_CONTRATO.DTINCLUSAO BETWEEN NVL(:DTINICIO, SYSDATE-30) AND NVL(:DTFINAL, SYSDATE)
                         AND FCT_CONTRATO.NRSEQCONTRATOPAI IS NOT NULL
                         AND FCT_CONTRATO.NRVERSAOCONTRATO > 1
                         AND FCT_CONTRATO.IDATIVO = 'S'
                       GROUP BY FCT_CONTRATO.NRTIPOORCAMENTO, FCT_TIPOORCAMENTO.NMTIPOORCAMENTO
                       ORDER BY QUANTIDADE DESC) GRAF
                       WHERE ROWNUM <= 7";

      $std = $this->connection->prepare($sql);
      $std->execute($parametros);
      return $std->fetchAll();
    }

    public function getContractsCloseToExpiration() {

      $sql = "SELECT NRSEQCONTRATO,
                   NRCONTRATOFCT,
                   NRPARCNEGOCIO,
                   NMSECUNDARIPARC,
                   DTFINVIGENCONT,
                   DTPADREAJCONT,
                   QTDIASVENCPRAZO,
                   CASE
                       WHEN QTDIASVENCPRAZO < 0 THEN
                           'Prazo venceu em ' || QTDIASVENCPRAZO * -1 || ' dias (' || DTFINVIGENCONT || ')'
                       WHEN QTDIASVENCPRAZO = 0 THEN
                           'Prazo vence hoje (' || DTFINVIGENCONT || ')'
                       ELSE
                           'Prazo vence em ' || QTDIASVENCPRAZO || ' dias (' || DTFINVIGENCONT || ')'
                   END AS STRPRAZO,
                   QTDIASVENCREAJ,
                   CASE
                       WHEN QTDIASVENCREAJ < 0 THEN
                           'Reajuste ' || QTDIASVENCREAJ * -1 || ' dias atrasado (' || DTPADREAJCONT || ')'
                       WHEN QTDIASVENCREAJ = 0 THEN
                           'Reajustar hoje (' || DTPADREAJCONT || ')'
                       ELSE
                           'Reajustar em ' || QTDIASVENCREAJ || ' dias (' || DTPADREAJCONT || ')'
                   END AS STRREAJ
              FROM (
               SELECT CONT.NRSEQCONTRATO,
                      CONT.NRCONTRATOFCT,
                      CONT.NRPARCNEGOCIO,
                      PARC.NMSECUNDARIPARC,
                      TO_CHAR(NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO), 'DD/MM/YYYY') AS DTFINVIGENCONT,
                      TO_CHAR(CONT.DTPADREAJCONT, 'DD/MM/YYYY') AS DTPADREAJCONT,
                      TRUNC(NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO)) - TRUNC(SYSDATE) AS QTDIASVENCPRAZO,
                      TRUNC(CONT.DTPADREAJCONT) - TRUNC(SYSDATE) AS QTDIASVENCREAJ
                 FROM FCT_CONTRATO CONT
                INNER JOIN PARCNEGOCIO PARC
                   ON CONT.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
                WHERE CONT.IDATIVO = 'S'
                  AND CONT.IDSITUACAOCONTRATO IN ('A', 'S')
                  AND (
                      (NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO) <= TRUNC(SYSDATE+30)) OR
                      (TRUNC(CONT.DTPADREAJCONT) <= TRUNC(SYSDATE+30))
                  )
                ORDER BY CONT.DTPADREAJCONT, NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO)
            )";

       return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getContractsPendingBilling($parametros) {

    // debugApi([$parametros]);

      $sql = "SELECT CONT.NRSEQCONTRATO,
                     CONT.NRCONTRATOFCT,
                     CONT.NRPARCNEGOCIO,
                     PARC.NMSECUNDARIPARC,
                     FAT.NRFATURACONT,
                     TO_CHAR(FAT.DTLIMITEAPU, 'DD/MM/YYYY') AS DTLIMITEAPU,
                     'Data Limite de Apuração - ' || TO_CHAR(FAT.DTLIMITEAPU, 'DD/MM/YYYY') AS STRDTLIMITE
                FROM FCT_FATURACONT FAT
               INNER JOIN FCT_RELACAGRUPCONTFAT REL
                  ON FAT.NRFATURACONT = REL.NRFATURACONT
               INNER JOIN FCT_CONTRATO CONT
                  ON REL.NRSEQCONTRATO = CONT.NRSEQCONTRATO
               INNER JOIN PARCNEGOCIO PARC
                  ON CONT.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
               WHERE FAT.IDSITUACAO IN ('1', '6')
                 AND TRUNC(FAT.DTLIMITEAPU) BETWEEN TRUNC(NVL(TO_DATE(:DTINI, 'DD/MM/YYYY'),SYSDATE-30)) AND TRUNC(NVL(TO_DATE(:DTFIN, 'DD/MM/YYYY'), SYSDATE))
               ORDER BY FAT.DTLIMITEAPU";

      return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);

    }
}